# Python imports

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

# Application imports
from .lsp_message_box import LSPMessageBox
from .enteries.alt_command_entry import AltCommandEntry
from .enteries.command_entry import CommandEntry
from .enteries.socket_entry import SocketEntry
from .enteries.init_options_source_view import InitOptionsSourceView
from .buttons.info_link_button import InfoLinkButton
from .log_list import LogList



class LSPUI(Gtk.Grid):
    def __init__(self, language, data):
        super(LSPUI, self).__init__()

        self._data     = data
        self._language = language

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show()


    def _setup_styling(self):
        ctx = self.get_style_context()
        ctx.add_class("lsp-ui")

        self.set_hexpand(True)
        self.set_vexpand(True)
        self.set_margin_top(5)
        self.set_margin_bottom(5)
        self.set_margin_left(10)
        self.set_margin_right(10)

    def _setup_signals(self):
        ...

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        scrolled_win1          = Gtk.ScrolledWindow()
        scrolled_win2          = Gtk.ScrolledWindow()
        self.log_list          = LogList()

        alt_command_lbl        = Gtk.Label(label="Alt Command:")
        command_lbl            = Gtk.Label(label="Command:")
        socket_lbl             = Gtk.Label(label="Socket:")
        init_options_lbl       = Gtk.Label(label="Initialization Options")
        message_box_lbl        = Gtk.Label(label="Message Box")
        log_list_lbl           = Gtk.Label(label="Logs")

        self.link_btn          = InfoLinkButton(self._language.title(), self._data["info"])
        self.alt_command_entry = AltCommandEntry( self._data["alt-command"] )
        self.command_entry     = CommandEntry( self._data["command"] )
        self.socket_entry      = SocketEntry( self._data["socket"] )
        self.init_ops_src_vw   = InitOptionsSourceView( self._data["initialization-options"] )
        self.message_box       = LSPMessageBox()

        init_options_lbl.set_margin_top(10)
        message_box_lbl.set_margin_top(10)
        log_list_lbl.set_margin_top(10)
        log_list_lbl.set_margin_bottom(10)
        scrolled_win2.set_size_request(320, -1)

        scrolled_win1.add(self.init_ops_src_vw)
        scrolled_win2.add(self.log_list)

        # child, left, top, width, height
        self.attach(self.link_btn,          0, 0, 3, 1)

        self.attach(alt_command_lbl,        0, 1, 1, 1)
        self.attach(command_lbl,            0, 2, 1, 1)
        self.attach(socket_lbl,             0, 3, 1, 1)

        self.attach(self.alt_command_entry, 1, 1, 2, 1)
        self.attach(self.command_entry,     1, 2, 2, 1)
        self.attach(self.socket_entry,      1, 3, 2, 1)

        self.attach(init_options_lbl,       0, 4, 3, 1)
        self.attach(scrolled_win1,          0, 5, 3, 1)

        self.attach(message_box_lbl,        0, 6, 3, 1)
        self.attach(self.message_box,       0, 7, 3, 1)

        self.attach(log_list_lbl,           3, 0, 3, 1)
        self.attach(scrolled_win2,          3, 1, 3, 11)

        for child in self.get_children():
            child.show()