# Python imports
import json
import enum

# Lib imports

# Application imports
from libs.dto.lsp_structs import TextDocumentItem



class MessageEncoder(json.JSONEncoder):
    """
    Encodes an object in JSON
    """

    def default(self, o):  # pylint: disable=E0202
        return o.__dict__



class LSPRequest(object):
    def __init__(self, id: int, method: str, params: dict):
        """
        Constructs a new LSP Request instance.

        :param int id: Message id to track instance.
        :param str method: The type of lsp request being made.
        :param dict params: The arguments of the given method. 
        """
        self.jsonrpc = "2.0"
        self.id      = id
        self.method  = method
        self.params  = params


class LSPNotification(object):
    def __init__(self, method: str, params: dict):
        """
        Constructs a new LSP Notification instance.

        :param str method: The type of lsp notification being made.
        :param dict params: The arguments of the given method. 
        """
        self.jsonrpc = "2.0"
        self.method  = method
        self.params  = params


class LSPResponse(object):
    def __init__(self, id: int or None, result: dict):
        """
        Constructs a new LSP Response instance.

        :param str method: The type of lsp notification being made.
        :param dict params: The arguments of the given method. 
        """
        self.jsonrpc = "2.0"
        self.id      = id
        self.result  = result
